#include <stdlib.h>

#include "Engine.h"
#include "Snake.h"

/* * *
 * De-allocate heap memory for the snake passed in.
 * * */
void Snake_Free(Snake_t** snake) {
	// Create pointers to probe and free each snake node.
	Snake_t* prob = *snake;
	Snake_t* toFree = NULL;
	// Loop through the snake - free every node.
	while (prob != NULL) {
		toFree = prob;
		prob = prob->next;
		free(toFree);
	}
}

/* * *
 * Allocate memory on the heap for snake with length passed in.
 * Initialize it with data from board passed in.
 * * */
void Snake_Init(const Board* board, Snake_t** snake, const unsigned int length) {
	// Create pointers for initializing the snake passed in.
	Snake_t* new  = NULL;
	Snake_t* next = NULL;
	// Loop through the length of the snake.
	// Allocate memor on the heap and assign value to each new node.
	for (unsigned int i = 0; i < length; i++) {
		// Allocate memor for new node on the heap.
		new = malloc(sizeof(Snake_t) * 1);
		// Check for memory error - free snake and no more action.
		if (new == NULL) {
			Snake_Free(snake);
			*snake = NULL;
			break;
		}
		// Assign value to the new node.
		new->vBody = i + 1;
		new->yLoc = (board->yMax/2) - (board->height/3);
		new->xLoc = (board->xMax/2) - (board->width/3) + i;
		new->next = next;
		// Reset next.
		next = new;
		// Check for last node - assign it to be the head of the snake.
		if (i == length - 1) {
			new->vHead = SNAKE_HEAD; // Value of the snake's head.
			*snake = new; // Assign to the snake passed in.
		}
	}
}

/* * *
 * Draw the whole snake inside the window of board.
 * * */
void Snake_Draw(const Board* board, Snake_t** snake) {
	// Creating a pointer to probe the first node of the snake.
	Snake_t* prob = *snake;
	// Loop through the whole snake and print their value respectively.
	while (prob != NULL) {
		(prob->vHead != SNAKE_HEAD) 
			? mvwprintw(board->win, prob->yLoc, prob->xLoc, "%u", prob->vBody)
			: mvwprintw(board->win, prob->yLoc, prob->xLoc, "%c", prob->vHead);
		// Move probe to the next node.
		prob = prob->next;
	}
	// Refresh window of board to update changes to user.
	wrefresh(board->win);
}

/* * *
 * Erase the whole snake inside the window of board.
 * * */
void Snake_Erase(const Board* board, Snake_t** snake) {
	// Creating a pointer to probe the first node (head) of the snake.
	Snake_t* prob = *snake;
	// Loop through the whole snake and print blank value.
	while (prob != NULL) {
		(prob->vHead != SNAKE_HEAD) 
			? mvwprintw(board->win, prob->yLoc, prob->xLoc, " ")
			: mvwprintw(board->win, prob->yLoc, prob->xLoc, " ");
		// Move probe to the next node.
		prob = prob->next;
	}
}

/* * *
 * Add a new node to the head of the snake with the coordinate value passed in.
 * * */
void Snake_Add(Snake_t** snake, unsigned int yLoc, unsigned int xLoc) {
	// Create a pointer to the old head node.
	Snake_t* old = *snake;
	// Disable the old head.
	old->vHead = 'a';
	// Allocate memory from heap for new node.
	Snake_t* new = malloc(sizeof(Snake_t) * 1);
	// Set coordinate from value passed in.
	new->yLoc = yLoc;
	new->xLoc = xLoc;
	// Set value for head.
	new->vHead = SNAKE_HEAD;
	new->next = old;
	// Get value for body from old.
	unsigned int value = old->vBody + 1;
	// Check for 2 digit number - set back to 1 digit.
	if (value >= 10) value = 0;
	new->vBody = value;
	// Set snake to new head node.
	*snake = new;
}

/* * *
 * Stepping the coordinate values of each node to the next along the whole snake.
 * * */
static void Snake_Step(Snake_t** snake) {
	// Create 2 nodes to transfer coordinates.
	Snake_t node1, node2;
	// Start probe from the head.
	Snake_t* prob = *snake;
	// Check for valid snake before copy the coordiantes from probe.
	if (prob != NULL) {
		// Copy coordinate from probe to first node.
		node1.xLoc = prob->xLoc;
		node1.yLoc = prob->yLoc;
		// Move probe to the next node.
		prob = prob->next;
		// Loop through every nodes and transfer their coordinate values.
		while (prob != NULL) {
			// Get coordinate from probe to second node.
			node2.xLoc = prob->xLoc;
			node2.yLoc = prob->yLoc;
			// Set coordinate from first node to probe.
			prob->yLoc = node1.yLoc;
			prob->xLoc = node1.xLoc;
			// Transfer coordinate for next round.
			node1.yLoc = node2.yLoc;
			node1.xLoc = node2.xLoc;
			// Move probe to the next node.
			prob = prob->next;
		}
	}
}

/* * *
 * Stepping the whole snake along before moving the head of snake left by one.
 * * */
void Snake_Move_Left(Snake_t** snake) {
	Snake_Step(snake);
	// Move the head.
	(*snake)->xLoc--;
}

/* * *
 * Stepping the whole snake along before moving the head of snake down by one.
 * * */
void Snake_Move_Down(Snake_t** snake) {
	Snake_Step(snake);
	// Move the head.
	(*snake)->yLoc++;
}

/* * *
 * Stepping the whole snake along before moving the head of snake up by one.
 * * */
void Snake_Move_Up(Snake_t** snake) {
	Snake_Step(snake);
	// Move the head.
	(*snake)->yLoc--;
}

/* * *
 * Stepping the whole snake along before moving the head of snake right by one.
 * * */
void Snake_Move_Right(Snake_t** snake) {
	Snake_Step(snake);
	// Move the head.
	(*snake)->xLoc++;
}

